#!/usr/bin/bash
#
#   Do initial configuration for the gramps-webapi
#

CURDIR=$(pwd)
CFG="config.cfg"
DATADIR="/usr/share/gramps-web"
EXECDIR="/usr/libexec/gramps-web"
TMPFILE=$(mktemp)
chmod 600 $TMPFILE

# Define the dialog exit status codes
: ${DIALOG_OK=0}
: ${DIALOG_CANCEL=1}
: ${DIALOG_HELP=2}
: ${DIALOG_EXTRA=3}
: ${DIALOG_ITEM_HELP=4}
: ${DIALOG_ESC=255}

HEIGHT=15
WIDTH=68

trap cleanup SIGINT SIGTERM SIGQUIT
cleanup () {
	# placeholder for any clean up needed
	clear
	echo "cleaning up..."
	rm -f $TMPFILE
}

# must run as superuser
uid=$(id -u)
if [ "$uid" != "0" ]
then
	echo "? $(basename $0) must be run as root."
	exit 1
fi

# work in the DATADIR
pushd $DATADIR 2>&1 > /dev/null

# create a random flask secret key
mkdir -p secret
python3 -c "import secrets;print(secrets.token_urlsafe(32))" | \
	tr -d "\n" > secret/secret
chmod 600 secret/secret
if [ -z "$TMP_SECRET_KEY" ]
then
	export TMP_SECRET_KEY=$(cat secret/secret)
fi

# collect config info
BACKTITLE="Gramps Web API Configuration"
BASE_URL="http://localhost"
SECRET_KEY="${TMP_SECRET_KEY}"

TREENAME="Empty Family Tree"
EMAIL_HOST="localhost"
EMAIL_PORT="25"
EMAIL_USE_TLS=False
USER_DB_URI="sqlite:////${DATADIR}/gramps_webapi_test.sqlite"
SEARCH_INDEX_DB_URI="sqlite:///${DATADIR}/indexdir/search_index.db"

dialog  --backtitle "$BACKTITLE" \
	--title "Web API Configuration" \
	--clear \
	--stdout \
	--output-separator '|' \
	--form "\nPlease enter desired configuration values:" 18 $WIDTH 10 \
	"Initial Family Tree Name:" 1 1 "$TREENAME" 1 28 25 0 \
	"Gramps Email Sent From:" 3 1 "gramps@localhost" 3 28 25 0 \
	"Outgoing Email Hostname:" 4 1 "$EMAIL_HOST" 4 28 25 0 \
	"Outgoing Email Port:" 5 1 "$EMAIL_PORT" 5 28 25 0 \
	"Outgoing Email Uses TLS?" 6 1 "true" 6 28 25 0 \
	"Outgoing Email Login Name:" 7 1 "" 7 28 25 0 \
	> $TMPFILE
return_value=$?
if [ $return_value != $DIALOG_OK ]
then
	clear
	echo "Canceling."
	exit 0
fi

echo tmpfile $(cat $TMPFILE) >> foo
treename=$(cat $TMPFILE | cut -d'|' -f1)
sendfrom=$(cat $TMPFILE | cut -d'|' -f2)
emailhost=$(cat $TMPFILE | cut -d'|' -f3)
emailport=$(cat $TMPFILE | cut -d'|' -f4)
usetls=$(cat $TMPFILE | cut -d'|' -f5)
emaillogin=$(cat $TMPFILE | cut -d'|' -f6)
echo treename \"$treename\" >> foo
echo sendfrom \"$sendfrom\" >> foo
echo emailhost \"$emailhost\" >> foo
echo emailport \"$emailport\" >> foo
echo usetls \"$usetls\" >> foo
echo emaillogin \"$emaillogin\" >> foo

dialog	--backtitle "$BACKTITLE" \
	--title "Outgoing Email Login" \
	--clear \
	--stdout \
	--passwordbox "\nEmail Login Password for $emaillogin:" $HEIGHT $WIDTH
	> $TMPFILE
emailpassword=$(cat $TMPFILE)
rm -f $TMPFILE
return_value=$?
if [ $return_value != $DIALOG_OK ]
then
	clear
	echo "Canceling."
	exit 0
fi

state=0
msg=""
while true
do
	case $state in
	0)	msg="\nCreating initial family tree ... "
		;;
	1)	msg="${msg}done\nCreating API config file ..."
		;;
	2)	msg="${msg}done\nCreating initial index file ..."
		;;
	3)	msg="${msg}done\nCreating owner/owner user  ..."
		;;
	4)	msg="${msg}done\nSystemd service available ..."
		;;
	5)	msg="${msg}done\n\nAll done."
		;;
	esac

	echo state $state, percent $percent >> foo
	echo $percent | dialog --backtitle "$BACKTITLE" \
		--title "Configuration Progress" \
		--gauge "$msg" $HEIGHT $WIDTH 0
	sleep 2
	case $state in
	0)	# create an empty testing family tree
		echo yes | gramps -r "$TREENAME" 2>/dev/null
		gramps -C "$TREENAME" -i data.gramps 2>/dev/null
		state=1
		;;

	1)	# build a config file
		(
		echo "TREE=\"${treename}\""
		echo "BASE_URL=\"http://localhost\""
		echo "SECRET_KEY=\"${TMP_SECRET_KEY}\""
		echo "USER_DB_URI=\"sqlite:////${DATADIR}/gramps_webapi_test.sqlite\""
		echo "SEARCH_INDEX_DB_URI=\"sqlite:///${DATADIR}/indexdir/search_index.db\""
		echo "EMAIL_HOST=\"$emailhost\""
		echo "EMAIL_PORT=\"$emailport\""
		echo "EMAIL_USE_TLS=\"$usetls\""
		echo "EMAIL_USER=\"$emaillogin\""
		echo "EMAIL_HOST_PASSWORD=\"$emailpassword\""
		echo "DEFAULT_FROM_EMAIL=\"gramps@localhost\""
		) > $CFG
		state=2
		;;

	2)	# create the search index if it doesn't already exist
		if [ -z "$(ls -A indexdir 2>/dev/null)" ]
		then
			python3 -m gramps_webapi --config $CFG \
				search index-full
		fi
		state=3
		;;

	3)	# create owner/owner user
		python3 -m gramps_webapi --config $CFG user add
			owner owner  --fullname Owner --role 4
		state=4
		;;

	4)	#-- install systemd service
		# nothing to do; it should have been put in the
		# proper place when the package was installed
		state=5
		;;

	*)	break
		;;
	esac
done
clear

